@@ -12,6 +12,7 @@ use proc_macro::TokenStream;
 use proc_macro_error::abort_call_site;
 use regex::Regex;
 use syn::{parse_macro_input, ItemFn, ItemMod};
+use sysinfo::{System, SystemExt};
 
 use crate::utils::{fn_macro, is_module, mod_macro};
 
@@ -563,3 +564,184 @@ fn check_user_condition(user_name: String) -> (bool, String) {
         format!("because this case should run with user {}", user_name),
     )
 }
+
+/// Run test case when memory size enough
+///
+/// ```
+/// #[cfg(test)]
+/// mod tests {
+///
+///     // Only works with enough memory size
+///     #[test_with::mem(100GB)]
+///     #[test]
+///     fn test_ignored() {
+///         panic!("should be ignored")
+///     }
+/// }
+/// ```
+#[proc_macro_attribute]
+pub fn mem(attr: TokenStream, stream: TokenStream) -> TokenStream {
+    if is_module(&stream) {
+        mod_macro(
+            attr,
+            parse_macro_input!(stream as ItemMod),
+            check_mem_condition,
+        )
+    } else {
+        fn_macro(
+            attr,
+            parse_macro_input!(stream as ItemFn),
+            check_mem_condition,
+        )
+    }
+}
+
+fn check_mem_condition(mem_size_str: String) -> (bool, String) {
+    let mut sys = System::new_all();
+    sys.refresh_all();
+    let mem_size = match byte_unit::Byte::from_str(format!("{} KB", sys.total_memory())) {
+        Ok(b) => b,
+        Err(_) => abort_call_site!("memory size description is not correct"),
+    };
+    let mem_size_limitation = match byte_unit::Byte::from_str(&mem_size_str) {
+        Ok(b) => b,
+        Err(_) => abort_call_site!("system memory size can not get"),
+    };
+    (
+        mem_size >= mem_size_limitation,
+        format!("because the memory less than {}", mem_size_str),
+    )
+}
+
+/// Run test case when swap size enough
+///
+/// ```
+/// #[cfg(test)]
+/// mod tests {
+///
+///     // Only works with enough swap size
+///     #[test_with::swap(100GB)]
+///     #[test]
+///     fn test_ignored() {
+///         panic!("should be ignored")
+///     }
+/// }
+/// ```
+#[proc_macro_attribute]
+pub fn swap(attr: TokenStream, stream: TokenStream) -> TokenStream {
+    if is_module(&stream) {
+        mod_macro(
+            attr,
+            parse_macro_input!(stream as ItemMod),
+            check_swap_condition,
+        )
+    } else {
+        fn_macro(
+            attr,
+            parse_macro_input!(stream as ItemFn),
+            check_swap_condition,
+        )
+    }
+}
+
+fn check_swap_condition(swap_size_str: String) -> (bool, String) {
+    let mut sys = System::new_all();
+    sys.refresh_all();
+    let swap_size = match byte_unit::Byte::from_str(format!("{} KB", sys.total_swap())) {
+        Ok(b) => b,
+        Err(_) => abort_call_site!("Swap size description is not correct"),
+    };
+    let swap_size_limitation = match byte_unit::Byte::from_str(&swap_size_str) {
+        Ok(b) => b,
+        Err(_) => abort_call_site!("Can not get system swap size"),
+    };
+    (
+        swap_size >= swap_size_limitation,
+        format!("because the swap less than {}", swap_size_str),
+    )
+}
+
+/// Run test case when cpu core enough
+///
+/// ```
+/// #[cfg(test)]
+/// mod tests {
+///
+///     // Only works with enough cpu core
+///     #[test_with::cpu_core(32)]
+///     #[test]
+///     fn test_ignored() {
+///         panic!("should be ignored")
+///     }
+/// }
+/// ```
+#[proc_macro_attribute]
+pub fn cpu_core(attr: TokenStream, stream: TokenStream) -> TokenStream {
+    if is_module(&stream) {
+        mod_macro(
+            attr,
+            parse_macro_input!(stream as ItemMod),
+            check_cpu_core_condition,
+        )
+    } else {
+        fn_macro(
+            attr,
+            parse_macro_input!(stream as ItemFn),
+            check_cpu_core_condition,
+        )
+    }
+}
+
+fn check_cpu_core_condition(core_limitation_str: String) -> (bool, String) {
+    (
+        match core_limitation_str.parse::<usize>() {
+            Ok(c) => num_cpus::get() >= c,
+            Err(_) => abort_call_site!("core limitation is incorrect"),
+        },
+        format!("because the cpu core less than {}", core_limitation_str),
+    )
+}
+
+/// Run test case when physical cpu core enough
+///
+/// ```
+/// #[cfg(test)]
+/// mod tests {
+///
+///     // Only works with enough cpu core
+///     #[test_with::phy_core(32)]
+///     #[test]
+///     fn test_ignored() {
+///         panic!("should be ignored")
+///     }
+/// }
+/// ```
+#[proc_macro_attribute]
+pub fn phy_core(attr: TokenStream, stream: TokenStream) -> TokenStream {
+    if is_module(&stream) {
+        mod_macro(
+            attr,
+            parse_macro_input!(stream as ItemMod),
+            check_cpu_core_condition,
+        )
+    } else {
+        fn_macro(
+            attr,
+            parse_macro_input!(stream as ItemFn),
+            check_phy_core_condition,
+        )
+    }
+}
+
+fn check_phy_core_condition(core_limitation_str: String) -> (bool, String) {
+    (
+        match core_limitation_str.parse::<usize>() {
+            Ok(c) => num_cpus::get_physical() >= c,
+            Err(_) => abort_call_site!("physical core limitation is incorrect"),
+        },
+        format!(
+            "because the physical cpu core less than {}",
+            core_limitation_str
+        ),
+    )
+}
